//+------------------------------------------------------------------+
//|                                                          TME.mq4 |
//|                                                              JYE |
//|                              Copyright  2010, Jean-Yves EHRHART |
//|                                          http://www.ehrhart.info |
//+------------------------------------------------------------------+
//|  Extended Trend Magic indicator                                  |
//|  with a buffer for the signal and another for the value          |
//+------------------------------------------------------------------+
#property copyright "Copyright  2010, Jean-Yves EHRHART"
#property link      "http://www.ehrhart.info"

#property indicator_chart_window
#property indicator_buffers 4
#property indicator_color1 Lime
#property indicator_width1 2
#property indicator_color2 Red
#property indicator_width2 2

//+------------------------------------------------------------------+

extern int CCI = 50;
extern int ATR = 5;

//+------------------------------------------------------------------+

double bufferUp[];
double bufferDn[];
double bufferSg[];   // Signal value
double buffer[];     // Magic Trend value

//+------------------------------------------------------------------+

int init()
{
   SetIndexStyle(0, DRAW_LINE, STYLE_SOLID, 2);
   SetIndexBuffer(0, bufferUp);
   SetIndexStyle(1, DRAW_LINE, STYLE_SOLID, 2);
   SetIndexBuffer(1, bufferDn);
   SetIndexBuffer(2, bufferSg);
   SetIndexBuffer(3, buffer);
   SetIndexLabel(0, "TM Up");     
   SetIndexLabel(1, "TM Down");
   SetIndexLabel(2, "Signal");   // -1 = Down, +1 = Up
   SetIndexLabel(3, "Trend Magic");

   IndicatorDigits(MarketInfo( Symbol(), MODE_DIGITS ));

   return (0);
}

//+------------------------------------------------------------------+

int deinit()
{
   return (0);
}

//+------------------------------------------------------------------+

int start()
{
   double thisCCI;
   double lastCCI;

   int counted_bars = IndicatorCounted();
   if (counted_bars < 0) return (-1);
   if (counted_bars > 0) counted_bars--;
   int limit = Bars - counted_bars;

   for (int i = limit; i >= 0; i--)
   {
      thisCCI = iCCI(NULL, 0, CCI, PRICE_TYPICAL, i);
      lastCCI = iCCI(NULL, 0, CCI, PRICE_TYPICAL, i + 1);
      
      bufferSg[i] = bufferSg[i + 1];

      // Let's make the link beetween buffers down and up for drawing purpose
      if (thisCCI >= 0 && lastCCI < 0)
      {
         bufferUp[i + 1] = bufferDn[i + 1];
         bufferSg[i] = +1.0;
      }
      if (thisCCI <= 0 && lastCCI > 0)
      {
         bufferDn[i + 1] = bufferUp[i + 1];
         bufferSg[i] = -1.0;
      }
      
      // Calculating the new value of the indicator
      if (thisCCI >= 0)
      {
         bufferDn[i] = EMPTY_VALUE;
         bufferUp[i] = Low[i] - iATR(NULL, 0, ATR, i);
         if (bufferUp[i] < bufferUp[i + 1])
         {
            bufferUp[i] = bufferUp[i + 1];
         }
      }
      else
      {
         if (thisCCI <= 0)
         {
            bufferUp[i] = EMPTY_VALUE;
            bufferDn[i] = High[i] + iATR(NULL, 0, ATR, i);
            if (bufferDn[i] > bufferDn[i + 1])
            {
               bufferDn[i] = bufferDn[i + 1];
            }
         }
      }
      
      // Filling the Trend Magic buffer with a unique value
      if( bufferSg[i] == 1.0 )
      {
         buffer[i] = bufferUp[i];
      }
      else
      {
         buffer[i] = bufferDn[i];
      }
      
   }

   return (0);
}

//+------------------------------------------------------------------+
//+------------------------------------------------------------------+